#ifndef PHONGMATERIAL_H
#define PHONGMATERIAL_H

#include "material.h"
#include "glm/vec3.hpp"

class Texture;

class PhongMaterial : public Material
{
private:
    enum TextureSlots
    {
        DIFFUSE_SLOT,
        AMBIENT_SLOT,
        NORMALS_SLOT,
        SPECULAR_SLOT,
        ALPHA_SLOT,
    };

    glm::vec3 ambient;
    glm::vec3 diffuse;
    glm::vec3 specular;
    float shininess;
    bool castShadow;
    Texture* ambient_texture;
    Texture* diffuse_texture;
    Texture* specular_texture;
    Texture* normal_map;
    Texture* alpha_mask;

public:

	PhongMaterial() :
        ambient(0),
		diffuse(0.5f),
		specular(0.5f),
		shininess(10),
        castShadow(true),
        ambient_texture(NULL),
        diffuse_texture(NULL),
        specular_texture(NULL),
        normal_map(NULL),
        alpha_mask(NULL)
    {}

    virtual ~PhongMaterial() {}

    virtual void bindAttributes(Shader* myShader = NULL);

    virtual unsigned int getFlags();

    void setAmbientTexture(Texture* myTexture) {ambient_texture = myTexture;}
    void setDiffuseTexture(Texture* myTexture) {diffuse_texture = myTexture;}
    void setSpecularTexture(Texture* myTexture) {specular_texture = myTexture;}
    void setNormalMap(Texture* myNormalMap) {normal_map = myNormalMap;}
    void setAlphaMask(Texture* myAlphaMask) {alpha_mask = myAlphaMask;}
};

#endif // PHONGMATERIAL_H
