#include "scene.h"
#include "mesh.h"
#include <set>


std::vector<unsigned int> Scene::getMeshTypes()
{
    std::set<unsigned int> typesSet;
    std::vector<unsigned int> types;
    for(SceneIterator<GeometryNode*>* geometryIt = scene->getGeometry();
        geometryIt->isValid(); geometryIt->next())
    {
        typesSet.emplace(geometryIt->getItem()->mesh->getFlags());
    }
    for(unsigned int type : typesSet)
        types.push_back(type);
    return types;
}

std::vector<unsigned int> Scene::getLightTypes()
{
    std::set<unsigned int> typesSet;
    std::vector<unsigned int> types;
    for(SceneIterator<Light*>* lightIt = scene->getLights();
        lightIt->isValid(); lightIt->next())
    {
        typesSet.emplace(getFlags(geometryIt->getItem()));
    }
    for(unsigned int type : typesSet)
        types.push_back(type);
    return types;
}

ArrayScene::~ArrayScene()
{
    clearScene();
}

void ArrayScene::addMesh(GeometryNode* node)
{
    Node n;
    n.geometry = node;
    unsigned int flags = node->mesh->getFlags();
    if(nodes.count(flags) > 0)
        nodes[m->getFlags()]->push_back(n);
    else
    {
        std::vector<Node> *vec = new std::vector<Node>();
        vec->push_back(n);
        nodes[m->getFlags()] = vec;
    }
}

void ArrayScene::addMesh(Mesh* m, glm::mat4 transform)
{
    GeometryNode *gn = new GeometryNode(m, transform);
    allocations.push_back(gn);
    addMesh(gn);
}

void ArrayScene::clearScene()
{
    for(auto it : nodes)
        delete it;
    for(GeometryNode *gn : allocations)
        delete gn;
    allocations.clear();
    nodes.clear();
}

SceneIterator<Light*> ArrayScene::getLights(unsigned int flags)
{
    if(nodes.count(flags))
        return ArrayIterator<Light*>(*(nodes[flags]));
}

SceneIterator<GeometryNode*> ArrayScene::getGeometry(unsigned int flags)
{
    if(nodes.count(flags))
        return ArrayIterator<GeometryNode*>(*(nodes[flags]));
}

